@@ -1,6 +1,10 @@
 <?php
 include_once(SM_PATH . 'plugins/proon/utils.php');
 
+// This file is mostly for action which take place as a
+// result of clicks on the proon form.  They're separated
+// out here mainly to declutter form.php.
+
 /**
  * This function queries an already-selected message folder
  * for information for all messages in it and pops the results
@@ -8,12 +12,12 @@ include_once(SM_PATH . 'plugins/proon/utils.php');
  * called $m.
  */
 function proon_get_messages_to_consider(&$m, &$imapStr) {
-  global $proon_log_verbose;
   proon_login($imapStr);
 
   $fetch = 'FETCH 1:* (UID FLAGS RFC822.SIZE INTERNALDATE)';
   $t1 = microtime();
   $result = sqimap_run_command_list($imapStr, $fetch, true/*false*/, $response, $message, true);
+  global $proon_log_verbose;
   $proon_log_verbose && proon_log_delta($fetch, $t1, null, $m[PROON_M_TARGET]);
   // I'm not entirely clear on why the results are nested in exactly this way,
   // so I hope it's good for all environments.
@@ -93,8 +97,7 @@ function proon_sort_conveniently(&$p, &$m, $sizespan, $countspan) {
   ||  $countspan != null  &&  $countspan != '') {
     // default is by date
     $sizebydate = true;
-    if (isset($p[PROON_P_OPTIONS][PROON_O_U_SC_ORDER])  
-    &&  $p[PROON_P_OPTIONS][PROON_O_U_SC_ORDER] == 'size') {
+    if (proon_effective_option($p, PROON_O_U_SC_ORDER)  == 'size') {
       $sizebydate = false;
     }
     if ($sizebydate) {
@@ -178,11 +181,11 @@ function proon_is_readwrite(&$rights) {
 
 /**
  * Given a target folder, get the list of messages, shuffle through
- * them according to pruing criteria, and truen the list of UIDs
+ * them according to pruning criteria, and return the list of UIDs
  * to prune from that folder.  The folder is still selected when
  * this method returns.  (We actually return this giant bucket called
  * $m which has all kinds of stuff in it besides the list of things
- * to prune.
+ * to prune.)
  */
 function proon_get_uids_one_folder(&$p, $target, $select_target, $spans, &$imapStr) {
 
@@ -206,10 +209,9 @@ function proon_get_uids_one_folder(&$p, $target, $select_target, $spans, &$imapS
 
   global $proon_readwrite_check;
 
-  sm_print_r('Rights', $mbx['RIGHTS']);
   if (!proon_is_readwrite($mbx['RIGHTS'])) return $m;
 
-  $toss_unseen = isset($p[PROON_F_UNSEENS][$target]);
+  $toss_unseen = proon_effective_folder_option($p, PROON_O_F_TOSS_UNSEEN, $target);
 
   proon_get_messages_to_consider($m, $imapStr);
   foreach ($m[PROON_M_FLAGS] as $uid => $flags) {
@@ -238,14 +240,14 @@ function proon_get_uids_one_folder(&$p, $target, $select_target, $spans, &$imapS
     }
   }
 
-  if (!isset($p[PROON_P_OPTIONS][PROON_O_U_DIS_DATE])) {
+  if (!proon_effective_option($p, PROON_O_U_DIS_DATE)) {
     proon_consider_date($p, $m, $spans[PROON_SPAN_DATE]);
   }
   proon_sort_conveniently($p, $m, $spans[PROON_SPAN_DATE], $spans[PROON_SPAN_COUNT]);
-  if (!isset($p[PROON_P_OPTIONS][PROON_O_U_DIS_SIZE])) {
+  if (!proon_effective_option($p, PROON_O_U_DIS_SIZE)) {
     proon_consider_size($p, $m, $spans[PROON_SPAN_SIZE]);
   }
-  if (!isset($p[PROON_P_OPTIONS][PROON_O_U_DIS_COUNT])) {
+  if (!proon_effective_option($p, PROON_O_U_DIS_COUNT)) {
     proon_consider_count($p, $m, $spans[PROON_SPAN_COUNT]);
   }
 
@@ -277,13 +279,9 @@ function proon_action_save(&$p, &$outcome, &$notice) {
   }
 
   foreach ($p[PROON_F_DATE_OR_SIZE_OR_COUNT] as $target => $junk) {
-    $rawdate = (isset($p[PROON_F_DATES][$target]) ? $p[PROON_F_DATES][$target] : '');
-    $rawsize = (isset($p[PROON_F_SIZES][$target]) ? $p[PROON_F_SIZES][$target] : '');
-    $rawcount = (isset($p[PROON_F_COUNTS][$target]) ? $p[PROON_F_COUNTS][$target] : '');
-    $spans = array();
-    $note = proon_check_spans($spans, $rawdate, $rawsize, $rawcount);
-    if ($note) {
-      $outcome[$target][PROON_OUTCOME_ERROR] = $note;
+    $spans = proon_get_spans($p, $target);
+    if (! is_array($spans)) {
+      $outcome[$target][PROON_OUTCOME_ERROR] = $spans;
       $ok_to_save = false;
     }
   }
@@ -333,8 +331,6 @@ function proon_transfer_outcome(&$m, &$a) {
   $a[PROON_OUTCOME_DUE_TO_SIZE] = $m[PROON_M_DUE_TO_SIZE];
   $a[PROON_OUTCOME_DUE_TO_COUNT] = $m[PROON_M_DUE_TO_COUNT];
   $a[PROON_OUTCOME_TOSSED_UNSEEN] = -1;
-  //sm_print_r('m', $m);
-  //sm_print_r('a', $a);
 }
 
 /**
@@ -353,6 +349,9 @@ function proon_action_now(&$p, &$target, &$outcome, &$imapStr) {
   proon_action_now_or_effect($p, $target, $outcome, $imapStr, true);
 }
 
+/**
+ * Common logic for "show effect" or "proon now" for a single folder.
+ */
 function proon_action_now_or_effect(&$p, &$target, &$outcome, &$imapStr, $doit) {
   if (strtoupper($target) == 'INBOX') {
     $select_target = 'INBOX';
@@ -366,16 +365,12 @@ function proon_action_now_or_effect(&$p, &$target, &$outcome, &$imapStr, $doit)
     proon_locale_on();
     $outcome[$target][PROON_OUTCOME_ERROR] = _("Folder doesn't exist.");
     proon_locale_off();
-    //sm_print_r('outcome', $outcome);
     return false;
   }
-  $rawdate = (isset($p[PROON_F_DATES][$target]) ? $p[PROON_F_DATES][$target] : '');
-  $rawsize = (isset($p[PROON_F_SIZES][$target]) ? $p[PROON_F_SIZES][$target] : '');
-  $rawcount = (isset($p[PROON_F_COUNTS][$target]) ? $p[PROON_F_COUNTS][$target] : '');
-  $spans = array();
-  $note = proon_check_spans($spans, $rawdate, $rawsize, $rawcount);
-  if ($note) {
-    $outcome[$target][PROON_OUTCOME_ERROR] = $note;
+
+  $spans = proon_get_spans($p, $target);
+  if (! is_array($spans)) {
+    $outcome[$target][PROON_OUTCOME_ERROR] = $spans;
     return false;
   }
 
@@ -390,7 +385,7 @@ function proon_action_now_or_effect(&$p, &$target, &$outcome, &$imapStr, $doit)
     if (count($m[PROON_M_TO_PRUNE]) > 0) {
       global $move_to_trash, $trash_folder, $folder_prefix;
       $old_move_to_trash = $move_to_trash;
-      $move_to_trash = (isset($p[PROON_P_OPTIONS][PROON_O_U_VIA_TRASH])  &&  $p[PROON_P_OPTIONS][PROON_O_U_VIA_TRASH]);
+      $move_to_trash = proon_effective_option($p, PROON_O_U_VIA_TRASH);
       $fptarget = $folder_prefix . $target;
       // This delete method includes an optional copy to trash first.
       proon_login($imapStr);
@@ -410,52 +405,54 @@ function proon_action_now_or_effect(&$p, &$target, &$outcome, &$imapStr, $doit)
   return true;
 }
 
-function proon_action_effect_or_prune_all(&$p, &$outcome, &$notice, &$imapStr, $doit) {
+/**
+ * Common logic for "show all effects" or "prune all".
+ */
+function proon_action_effect_or_prune_all(&$p, &$outcome, &$notice, &$imapStr, $doit, $manual) {
+  $t1 = microtime();
   $pruneall_notice = false;
-  $prune_via_trash = (isset($p[PROON_P_OPTIONS][PROON_O_U_VIA_TRASH])  &&  $p[PROON_P_OPTIONS][PROON_O_U_VIA_TRASH]);
+  $prune_via_trash = proon_effective_option($p, PROON_O_U_VIA_TRASH);
+  $trash_order = PROON_O_U_T_NATURAL;
   if ($prune_via_trash) {
-    if (isset($p[PROON_P_OPTIONS][PROON_O_U_TRASH_TIME])) {
-      $trash_order = $p[PROON_P_OPTIONS][PROON_O_U_TRASH_TIME];
-    }
-    else {
-      $trash_order = PROON_O_U_T_FIRST;
-    }
-  }
-  else {
-    $trash_order = PROON_O_U_T_NATURAL;
+    $trash_order = proon_effective_option($p, PROON_O_U_TRASH_TIME);
+    if (!$trash_order) $trash_order = PROON_O_U_T_FIRST;
   }
   global $trash_folder, $folder_prefix;
   $tbox = isset($trash_folder) ? $trash_folder : '';
   if ($tbox != ''  &&  isset($folder_prefix)  &&  strlen($folder_prefix) > 0) {
     $tbox = substr($tbox, strlen($folder_prefix));
   }
-  if ($trash_order == PROON_O_U_T_FIRST  &&  $tbox != '') {
+  $tbox_has_spans = proon_minimum_folder_option($p, PROON_O_F_DATE_SPAN, $tbox, 'd')
+                 || proon_minimum_folder_option($p, PROON_O_F_SIZE_SPAN, $tbox, 's')
+                 || proon_minimum_folder_option($p, PROON_O_F_COUNT_SPAN, $tbox, 'c');
+
+  $tbox_manual_only = proon_effective_folder_option($p, PROON_O_F_MANUAL_ONLY, $tbox);
+
+  if ($trash_order == PROON_O_U_T_FIRST  
+  &&  $tbox != ''  
+  &&  $tbox_has_spans  
+  &&  ($manual  ||  !$tbox_manual_only)) {
     if (false === proon_action_now_or_effect($p, $tbox, $outcome, $imapStr, $doit)) {
       $pruneall_notice = true;
     }
   }
-  $consider_unsubs = isset($p[PROON_P_OPTIONS][PROON_O_U_UNSUBSCRIBED]);
+  $consider_unsubs = proon_effective_option($p, PROON_O_U_UNSUBSCRIBED);
   foreach ($p[PROON_F_DATE_OR_SIZE_OR_COUNT] as $target => $junk) {
     if ($target == $tbox  &&  $trash_order != PROON_O_U_T_NATURAL) continue;
     $issubbed = proon_is_subscribed($p, $target, $imapStr);
-    if (! $issubbed) continue;
-    //sm_print_r('s', $target);
+    if (! $issubbed  &&  ! $consider_unsubs) continue;
+    $target_manual_only = proon_effective_folder_option($p, PROON_O_F_MANUAL_ONLY, $target);
+    if (! $manual  &&  $target_manual_only) continue;
+    if (!proon_mailbox_exists($p, $target, $imapStr)) continue;
     if (false === proon_action_now_or_effect($p, $target, $outcome, $imapStr, $doit)) {
       $pruneall_notice = true;
     }
   }
-  if ($consider_unsubs) {
-    foreach ($p[PROON_F_DATE_OR_SIZE_OR_COUNT] as $target => $junk) {
-      if ($target == $tbox  &&  $trash_order != PROON_O_U_T_NATURAL) continue;
-      $issubbed = proon_is_subscribed($p, $target, $imapStr);
-      if ($issubbed) continue;
-      //sm_print_r('u', $target);
-      if (false === proon_action_now_or_effect($p, $target, $outcome, $imapStr, $doit)) {
-	$pruneall_notice = true;
-      }
-    }
-  }
-  if ($trash_order == PROON_O_U_T_LAST  &&  $tbox != '') {
+
+  if ($trash_order == PROON_O_U_T_LAST
+  &&  $tbox != ''  
+  &&  $tbox_has_spans  
+  &&  ($manual  ||  !$tbox_manual_only)) {
     if (false === proon_action_now_or_effect($p, $tbox, $outcome, $imapStr, $doit)) {
       $pruneall_notice = true;
     }
@@ -463,9 +460,27 @@ function proon_action_effect_or_prune_all(&$p, &$outcome, &$notice, &$imapStr, $
   $notice = false;
   if ($doit  &&  $pruneall_notice) {
     proon_locale_on();
-    $notice = _("Some folders WERE NOT pruned due to improper date, size, or count spans.  See the folder list below for details.  Those without problems WERE pruned.");
+    $notice = _("Some folders WERE NOT pruned due to improper date, size, or count spans, or possibly other problems.  See the folder list below for details.  Those without problems WERE pruned.");
     proon_locale_off();
   }
+
+  $pairs = array();
+  foreach($outcome as $folder => $f_out) {
+    $dp = $f_out[PROON_OUTCOME_DUE_TO_DATE];
+    $sp = $f_out[PROON_OUTCOME_DUE_TO_SIZE];
+    $cp = $f_out[PROON_OUTCOME_DUE_TO_COUNT];
+    $tot_pruned = $dp + $sp + $cp;
+    if ($tot_pruned > 0) {
+      $pairs[$folder] = $tot_pruned;
+    }
+  }
+  $ep = ($doit ? 'PRUNE' : 'EFFECT');
+  $ma = ($manual ? 'MANUAL' : 'AUTO');
+  $pd = proon_to_pref($pairs);
+  if (! $pd) {
+    $pd = 'NONE';
+  }
+  proon_log_delta("PRUNE ALL, doit=$ep, manual=$ma, results=$pd", $t1, null, null);
   return;
 }
 
@@ -474,7 +489,8 @@ function proon_action_effect_or_prune_all(&$p, &$outcome, &$notice, &$imapStr, $
  * folders.  Individual folder pruning parameters are validated first.
  */
 function proon_action_alleffects(&$p, &$outcome, &$notice, &$imapStr) {
-  return proon_action_effect_or_prune_all($p, $outcome, $notice, $imapStr, false);
+  $ret = proon_action_effect_or_prune_all($p, $outcome, $notice, $imapStr, false, true);
+  return $ret;
 }
 
 /**
@@ -483,8 +499,9 @@ function proon_action_alleffects(&$p, &$outcome, &$notice, &$imapStr) {
  * Folders with good parameters get pruned and others get flagged as
  * not pruned.
  */
-function proon_action_pruneall(&$p, &$outcome, &$notice, &$imapStr) {
-  return proon_action_effect_or_prune_all($p, $outcome, $notice, $imapStr, true);
+function proon_action_pruneall(&$p, &$outcome, &$notice, &$imapStr, $manual) {
+  $ret = proon_action_effect_or_prune_all($p, $outcome, $notice, $imapStr, true, $manual);
+  return $ret;
 }
 
 ?>
