@@ -5,6 +5,10 @@ if (file_exists(SM_PATH . 'plugins/proon/site-config.php')) {
   include(SM_PATH . 'plugins/proon/site-config.php');
 }
 
+/**
+ * Utility for switching to the proon text domains.  It 
+ * has a limited push/pop-like capability.
+ */
 function proon_locale_on($s = false) {
   global $proon_log_verbose;
   $s && $proon_log_verbose && proon_log('L ON  ' . $s);
@@ -18,6 +22,10 @@ function proon_locale_on($s = false) {
   textdomain('proon');
 }
 
+/**
+ * Switches out of the proon text domain to whatever it was
+ * before that.
+ */
 function proon_locale_off($s = false) {
   global $proon_old_textdomain, $proon_log_verbose;
   $s && $proon_log_verbose && proon_log('L OFF ' . $s);
@@ -32,7 +40,6 @@ function proon_locale_off($s = false) {
   textdomain($proon_old_textdomain);
 }
 
-
 include_once(SM_PATH . 'plugins/proon/define.php');
 include_once(SM_PATH . 'plugins/proon/defineui.php');
 
@@ -73,7 +80,7 @@ function proon_to_pref(&$some_array) {
   $d3 = 'abcdefghijklmnopqrstuvwxyz';
   $d4 = '0123456789';
   $delim = proon_pick_delimiter($some_array, $d1 . $d2 . $d3 . $d4);
-  if ($delim == null) return false;
+  if ($delim == null) return false;  // this is very bad :-(
   $pref = '';
   if ($some_array) foreach($some_array as $key=>$value) {
     $pref .= "$delim$key$delim$value";
@@ -141,7 +148,26 @@ function proon_logout(& $imapStr) {
   $imapStr = null;
 }
 
-function proon_check_spans(&$spans, $rawdate, $rawsize, $rawcount) {
+/**
+ * Factor out some tedious logic for gathering the various per-folder user
+ * and site values for pruning spans.
+ */
+function proon_get_spans(&$p, &$target) {
+  $rawdate = (isset($p[PROON_O_F_DATE_SPAN][$target]) ? $p[PROON_O_F_DATE_SPAN][$target] : '');
+  $rawsize = (isset($p[PROON_O_F_SIZE_SPAN][$target]) ? $p[PROON_O_F_SIZE_SPAN][$target] : '');
+  $rawcount = (isset($p[PROON_O_F_COUNT_SPAN][$target]) ? $p[PROON_O_F_COUNT_SPAN][$target] : '');
+  $m = proon_check_spans($rawdate, $rawsize, $rawcount);
+  if ($m !== true) return $m;
+  $spans[PROON_SPAN_DATE] = proon_minimum_folder_option($p, PROON_O_F_DATE_SPAN, $target, 'd');
+  $spans[PROON_SPAN_SIZE] = proon_minimum_folder_option($p, PROON_O_F_SIZE_SPAN, $target, 's');
+  $spans[PROON_SPAN_COUNT] = proon_minimum_folder_option($p, PROON_O_F_COUNT_SPAN, $target, 'c');;
+  return $spans;
+}
+
+/**
+ * Checks the span values for syntactic correctness.
+ */
+function proon_check_spans($rawdate, $rawsize, $rawcount) {
   if (!$rawdate  &&  !$rawsize  &&  !$rawcount) {
     proon_locale_on();
     $m = _("None of the span values has been set for this folder.");
@@ -170,12 +196,12 @@ function proon_check_spans(&$spans, $rawdate, $rawsize, $rawcount) {
     proon_locale_off();
     return $m;
   }
-  $spans[PROON_SPAN_DATE] = $datespan;
-  $spans[PROON_SPAN_SIZE] = $sizespan;
-  $spans[PROON_SPAN_COUNT] = $countspan;
-  return null;
+  return true;
 }
-
+/**
+ * Converts a string date span to a number of seconds.
+ * Returns false if it doesn't like something.
+ */
 function proon_date_in_seconds($rawspan) {
   $rawspan = trim($rawspan);
   if (!isset($rawspan)  ||  $rawspan == ''  ||  $rawspan == '/') return false;
@@ -203,17 +229,32 @@ function proon_date_in_seconds($rawspan) {
   return $spansecs;
 }
 
+/**
+ * Converts a string size span to a number of bytes.
+ * Returns false if it doesn't like something.
+ */
 function proon_size_in_bytes($rawsize) {
   return proon_scaled_singles($rawsize, 'm');
 }
 
+/**
+ * Converts a string count span to a number of messages.
+ * Returns false if it doesn't like something.
+ */
 function proon_count_in_each($rawsize) {
   return proon_scaled_singles($rawsize, 'b');
 }
 
+/**
+ * Factored out common logic for conversion of count
+ * and size spans.  The scale suffixes are the same, but
+ * the default suffix is different.
+ * Returns false if it doesn't like something.
+ */
 function proon_scaled_singles($rawvalue, $default_suffix) {
+  if (!isset($rawvalue)) return false;
   $rawvalue = trim($rawvalue);
-  if (!isset($rawvalue) || $rawvalue == '') return false;
+  if ($rawvalue == '') return false;
   $suffix = substr($rawvalue, strlen($rawvalue) - 1);
   if (is_numeric($suffix)) {
     $number = $rawvalue;
@@ -238,15 +279,25 @@ function proon_scaled_singles($rawvalue, $default_suffix) {
   return $tally;
 }
 
+/**
+ * Makes a foldername safe for use inside HTML
+ */
+function proon_encode_foldername($folder) {
+  return base64_encode($folder);
+}
+
+/**
+ * Reverse of proon_encode_foldername
+ */
 function proon_pull_foldername($key) {
   list($word, $foldername) = explode('_', $key);
   return base64_decode($foldername);
 }
 
-function proon_encode_foldername($folder) {
-  return base64_encode($folder);
-}
-
+/**
+ * A utility for comparing two arrays to see if they differ
+ * in a way that proon cares about.
+ */
 function proon_arrays_differ(&$a, &$b) {
   if ($a === $b) return false;
   if (is_array($a)  &&  !is_array($b)) return true;
@@ -268,24 +319,32 @@ function proon_arrays_differ(&$a, &$b) {
 function proon_set_prefs(&$p) {
   global $data_dir, $username;
 
-  $pref = proon_to_pref($p[PROON_F_DATES]);
+  $pref = proon_to_pref($p[PROON_O_F_DATE_SPAN]);
   if ($pref == '') $pref = PROON_PREF_BLANK;
   setPref($data_dir, $username, PROON_O_F_DATE_SPAN, $pref);
 
-  $pref = proon_to_pref($p[PROON_F_SIZES]);
+  $pref = proon_to_pref($p[PROON_O_F_SIZE_SPAN]);
   if ($pref == '') $pref = PROON_PREF_BLANK;
   setPref($data_dir, $username, PROON_O_F_SIZE_SPAN, $pref);
 
-  $pref = proon_to_pref($p[PROON_F_COUNTS]);
+  $pref = proon_to_pref($p[PROON_O_F_COUNT_SPAN]);
   if ($pref == '') $pref = PROON_PREF_BLANK;
   setPref($data_dir, $username, PROON_O_F_COUNT_SPAN, $pref);
 
-  $pref = proon_to_pref($p[PROON_F_UNSEENS]);
+  $pref = proon_to_pref($p[PROON_O_F_TOSS_UNSEEN]);
   if ($pref == '') $pref = PROON_PREF_BLANK;
   setPref($data_dir, $username, PROON_O_F_TOSS_UNSEEN, $pref);
 
+  $pref = proon_to_pref($p[PROON_O_F_MANUAL_ONLY]);
+  if ($pref == '') $pref = PROON_PREF_BLANK;
+  setPref($data_dir, $username, PROON_O_F_MANUAL_ONLY, $pref);
+
   $proonopts = $p[PROON_P_OPTIONS];
 
+  $opt = PROON_O_U_LOGIN_N;
+  $pref = (isset($proonopts[$opt])) ? $proonopts[$opt] : PROON_PREF_BLANK;
+  setPref($data_dir, $username, $opt, $pref);
+
   $opt = PROON_O_U_PRUNE_INTERVAL;
   $pref = (isset($proonopts[$opt])) ? $proonopts[$opt] : PROON_PREF_BLANK;
   setPref($data_dir, $username, $opt, $pref);
@@ -336,108 +395,134 @@ function proon_set_prefs(&$p) {
 
 }
 
+/**
+ * Factored out common logic for fetching a single user preference item.
+ * Includes getting the equivalent (same name) site preference item.
+ */
+function proon_get_single_pref($opt, &$saw_pref_count, &$proonopts, &$proonsite) {
+  global $data_dir, $username;
+  $val = getPref($data_dir, $username, $opt);
+  if ($val) ++$saw_pref_count;
+  if ($val  &&  (string)$val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
+  global $$opt;
+  $val = $$opt;
+  if ($val  &&  (string)$val != PROON_PREF_BLANK) $proonsite[$opt] = $val;
+}
+
+/**
+ * Factored out common logic for fetching a per-folder user preference item.
+ * Includes getting the equivalent (same name) site preference item.
+ */
+function proon_get_list_pref($opt, &$saw_pref_count, &$p, &$proonsite) {
+  global $data_dir, $username;
+  $blob = getPref($data_dir, $username, $opt);
+  if ($blob) ++$saw_pref_count;
+  $p[$opt] = ($blob  &&  (string)$blob != PROON_PREF_BLANK) ? proon_from_pref($blob) : array();
+  global $$opt;
+  $blob = $$opt;
+  $proonsite[$opt] = ($blob  &&  (String)$blob != PROON_PREF_BLANK) ? proon_from_pref($blob) : array();
+}
+
 /**
  * Fetch all proon-related items from the user's preferences store.
  */
 function proon_get_prefs() {
   global $data_dir, $username;
-  $saw_pref_count = 0; // not used, but tells if at least one pref was read
+  $saw_pref_count = 0; // tells if at least one pref was read
+  $proonopts = array();
+  $proonsite = array();
 
-  $blob = getPref($data_dir, $username, PROON_O_F_DATE_SPAN);
-  if ($blob) ++$saw_pref_count;
-  $p[PROON_F_DATES] = ($blob  &&  $blob != PROON_PREF_BLANK) ? proon_from_pref($blob) : array();
-
-  $blob = getPref($data_dir, $username, PROON_O_F_SIZE_SPAN);
-  if ($blob) ++$saw_pref_count;
-  $p[PROON_F_SIZES] = ($blob  &&  $blob != PROON_PREF_BLANK) ? proon_from_pref($blob) : array();
-
-  $blob = getPref($data_dir, $username, PROON_O_F_COUNT_SPAN);
-  if ($blob) ++$saw_pref_count;
-  $p[PROON_F_COUNTS] = ($blob  &&  $blob != PROON_PREF_BLANK) ? proon_from_pref($blob) : array();
-
-  $blob = getPref($data_dir, $username, PROON_O_F_TOSS_UNSEEN);
-  if ($blob) ++$saw_pref_count;
-  $p[PROON_F_UNSEENS] = ($blob  &&  $blob != PROON_PREF_BLANK) ? proon_from_pref($blob) : array();
+  proon_get_list_pref(PROON_O_F_DATE_SPAN, $saw_pref_count, $p, $proonsite);
+  proon_get_list_pref(PROON_O_F_SIZE_SPAN, $saw_pref_count, $p, $proonsite);
+  proon_get_list_pref(PROON_O_F_COUNT_SPAN, $saw_pref_count, $p, $proonsite);
+  proon_get_list_pref(PROON_O_F_TOSS_UNSEEN, $saw_pref_count, $p, $proonsite);
+  proon_get_list_pref(PROON_O_F_MANUAL_ONLY, $saw_pref_count, $p, $proonsite);
 
   $dateorsizeorcountlist = array();
-  foreach ($p[PROON_F_DATES] as $folder => $value) {
+  foreach ($p[PROON_O_F_DATE_SPAN] as $folder => $value) {
     $dateorsizeorcountlist[$folder] = $value;
   }
-  foreach ($p[PROON_F_SIZES] as $folder => $value) {
+  foreach ($p[PROON_O_F_SIZE_SPAN] as $folder => $value) {
+    $dateorsizeorcountlist[$folder] = $value;
+  }
+  foreach ($p[PROON_O_F_COUNT_SPAN] as $folder => $value) {
     $dateorsizeorcountlist[$folder] = $value;
   }
   ksort($dateorsizeorcountlist);  // cosmetic
   $p[PROON_F_DATE_OR_SIZE_OR_COUNT] = $dateorsizeorcountlist;
   
-
-  $proonopts = array();
-
-  $opt = PROON_O_U_PRUNE_INTERVAL;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_VIA_TRASH;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_TRASH_TIME;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_UNSUBSCRIBED;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_DIS_DATE;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_DIS_SIZE;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_DIS_COUNT;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_SC_ORDER;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_MESSAGE_AFTER;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_SCREEN_AFTER;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_DIS_COLOR;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
-
-  $opt = PROON_O_U_PRUNE_LINK;
-  $val = getPref($data_dir, $username, $opt);
-  if ($val) ++$saw_pref_count;
-  if ($val  &&  $val != PROON_PREF_BLANK) $proonopts[$opt] = $val;
+  proon_get_single_pref(PROON_O_U_LOGIN_N, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_PRUNE_INTERVAL, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_VIA_TRASH, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_TRASH_TIME, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_UNSUBSCRIBED, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_DIS_DATE, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_DIS_SIZE, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_DIS_COUNT, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_SC_ORDER, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_MESSAGE_AFTER, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_SCREEN_AFTER, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_DIS_COLOR, $saw_pref_count, $proonopts, $proonsite);
+  proon_get_single_pref(PROON_O_U_PRUNE_LINK, $saw_pref_count, $proonopts, $proonsite);
 
   $p[PROON_P_OPTIONS]  = $proonopts;
+  $p[PROON_P_SITE]  = $proonsite;
   $p[PROON_P_PREFS_SEEN] = $saw_pref_count;
   
   return $p;
 }
 
+
+/**
+ * Return the effective value of some named option.  If
+ * there is a site pref, it rulez.  Else, return the user
+ * pref value.
+ */
+function proon_effective_option(&$p, $opt) {
+  $site_pref = (isset($p[PROON_P_SITE][$opt])  &&  $p[PROON_P_SITE][$opt]) ? ($p[PROON_P_SITE][$opt]) : null;
+  if ($site_pref) return $site_pref;
+  $user_pref = (isset($p[PROON_P_OPTIONS][$opt])  &&  $p[PROON_P_OPTIONS][$opt]) ? ($p[PROON_P_OPTIONS][$opt]) : null;
+  return $user_pref;
+}
+
+/**
+ * Return the effective value of some named per-folder option.  If
+ * there is a site pref, it rulez.  Else, return the user
+ * pref value.
+ */
+function proon_effective_folder_option(&$p, $opt, $target) {
+  $site_pref = (isset($p[PROON_P_SITE][$opt][$target])  &&  $p[PROON_P_SITE][$opt][$target]) ? ($p[PROON_P_SITE][$opt][$target]) : null;
+  if ($site_pref) return $site_pref;
+  $user_pref = (isset($p[$opt][$target])  &&  $p[$opt][$target]) ? ($p[$opt][$target]) : null;
+  return $user_pref;
+}
+
+/**
+ * Return the effective value of some named per-folder option.  It's the
+ * minimum of the site value and the user value (though null/zero/false
+ * for either of those is ignored).
+ */
+function proon_minimum_folder_option(&$p, $opt, $target, $type) {
+  $site_pref = (isset($p[PROON_P_SITE][$opt][$target])  &&  $p[PROON_P_SITE][$opt][$target]) ? ($p[PROON_P_SITE][$opt][$target]) : null;
+  $user_pref = (isset($p[$opt][$target])  &&  $p[$opt][$target]) ? ($p[$opt][$target]) : null;
+  if ($type == 'd') {
+    $site_pref = proon_date_in_seconds($site_pref);
+    $user_pref = proon_date_in_seconds($user_pref);
+  }
+  else if ($type == 's') {
+    $site_pref = proon_size_in_bytes($site_pref);
+    $user_pref = proon_size_in_bytes($user_pref);
+  }
+  else if ($type == 'c') {
+    $site_pref = proon_count_in_each($site_pref);
+    $user_pref = proon_count_in_each($user_pref);
+  }
+  if      (! $site_pref) return $user_pref;
+  else if (! $user_pref) return $site_pref;
+  if ($site_pref < $user_pref) return $site_pref;
+  return $user_pref;
+}
+
 /**
  * Read preference values from storage and compare to copies
  * received from form submission.  If any don't exactly match,
@@ -448,13 +533,18 @@ function proon_get_prefs() {
  */
 function proon_is_dirty(&$p) {
   $pref_p = proon_get_prefs();
-  if (proon_arrays_differ($p[PROON_F_DATES], $pref_p[PROON_F_DATES])) return true;
-  if (proon_arrays_differ($p[PROON_F_SIZES], $pref_p[PROON_F_SIZES])) return true;
-  if (proon_arrays_differ($p[PROON_F_COUNTS], $pref_p[PROON_F_COUNTS])) return true;
-  if (proon_arrays_differ($p[PROON_F_UNSEENS], $pref_p[PROON_F_UNSEENS])) return true;
+  if (proon_arrays_differ($p[PROON_O_F_DATE_SPAN], $pref_p[PROON_O_F_DATE_SPAN])) return true;
+  if (proon_arrays_differ($p[PROON_O_F_SIZE_SPAN], $pref_p[PROON_O_F_SIZE_SPAN])) return true;
+  if (proon_arrays_differ($p[PROON_O_F_COUNT_SPAN], $pref_p[PROON_O_F_COUNT_SPAN])) return true;
+  if (proon_arrays_differ($p[PROON_O_F_TOSS_UNSEEN], $pref_p[PROON_O_F_TOSS_UNSEEN])) return true;
+  if (proon_arrays_differ($p[PROON_O_F_MANUAL_ONLY], $pref_p[PROON_O_F_MANUAL_ONLY])) return true;
   return proon_arrays_differ($p[PROON_P_OPTIONS], $pref_p[PROON_P_OPTIONS]);
 }
 
+/**
+ * A little utility for calculating some widths to make reports
+ * (after pruning) pretty instead of ragged.
+ */
 function proon_calc_report_widths(&$p, &$outcome, $verbose) {
   $w = array();
   $w['duelabel'] = max(strlen(PROON_UI_DATE_SPAN), 
@@ -469,10 +559,10 @@ function proon_calc_report_widths(&$p, &$outcome, $verbose) {
   proon_locale_off();
   foreach ($outcome as $f => $an) {
     $w['fname'] = max($w['fname'], strlen(proon_folder_blat_name($f)));
-    $d = isset($p[PROON_F_DATES][$f]) ? $p[PROON_F_DATES][$f] : '';
-    $s = isset($p[PROON_F_SIZES][$f]) ? $p[PROON_F_SIZES][$f] : '';
-    $c = isset($p[PROON_F_COUNTS][$f]) ? $p[PROON_F_COUNTS][$f] : '';
-    $u = isset($p[PROON_F_UNSEENS][$f]) ? $yess : '';
+    $d = isset($p[PROON_O_F_DATE_SPAN][$f]) ? $p[PROON_O_F_DATE_SPAN][$f] : '';
+    $s = isset($p[PROON_O_F_SIZE_SPAN][$f]) ? $p[PROON_O_F_SIZE_SPAN][$f] : '';
+    $c = isset($p[PROON_O_F_COUNT_SPAN][$f]) ? $p[PROON_O_F_COUNT_SPAN][$f] : '';
+    $u = isset($p[PROON_O_F_TOSS_UNSEEN][$f]) ? $yess : '';
     $w['spanval'] = max($w['spanval'], strlen($d), strlen($s), strlen($c), strlen($u));
     $dp = $an[PROON_OUTCOME_DUE_TO_DATE];
     $sp = $an[PROON_OUTCOME_DUE_TO_SIZE];
@@ -483,6 +573,10 @@ function proon_calc_report_widths(&$p, &$outcome, $verbose) {
   return $w;
 }
 
+/**
+ * Create a stylish and handsome post-pruning report.  The on-screen
+ * report is verbose, but the email report isn't.
+ */
 function proon_format_outcome(&$p, &$outcome, $verbose) {
   $w = proon_calc_report_widths($p, $outcome, $verbose);
   if (! $verbose  &&  $w['total'] < 1) return null;
@@ -494,10 +588,10 @@ function proon_format_outcome(&$p, &$outcome, $verbose) {
   $yess = _("yes");
   proon_locale_off();
   foreach ($outcome as $f => $f_out) {
-    $d = isset($p[PROON_F_DATES][$f]) ? $p[PROON_F_DATES][$f] : '';
-    $s = isset($p[PROON_F_SIZES][$f]) ? $p[PROON_F_SIZES][$f] : '';
-    $c = isset($p[PROON_F_COUNTS][$f]) ? $p[PROON_F_COUNTS][$f] : '';
-    $u = isset($p[PROON_F_UNSEENS][$f]) ? $p[PROON_F_UNSEENS][$f] : '';
+    $d = isset($p[PROON_O_F_DATE_SPAN][$f]) ? $p[PROON_O_F_DATE_SPAN][$f] : '';
+    $s = isset($p[PROON_O_F_SIZE_SPAN][$f]) ? $p[PROON_O_F_SIZE_SPAN][$f] : '';
+    $c = isset($p[PROON_O_F_COUNT_SPAN][$f]) ? $p[PROON_O_F_COUNT_SPAN][$f] : '';
+    $u = isset($p[PROON_O_F_TOSS_UNSEEN][$f]) ? $p[PROON_O_F_TOSS_UNSEEN][$f] : '';
     $dp = $f_out[PROON_OUTCOME_DUE_TO_DATE];
     $sp = $f_out[PROON_OUTCOME_DUE_TO_SIZE];
     $cp = $f_out[PROON_OUTCOME_DUE_TO_COUNT];
@@ -531,6 +625,10 @@ function proon_format_outcome(&$p, &$outcome, $verbose) {
   return $report;
 }
 
+/**
+ * Make a folder name suitable for display.  This includes localizing
+ * INBOX or decoding other mailbox names.
+ */
 function proon_folder_blat_name($folder) {
   if (strcasecmp('INBOX', $folder) == 0) {
     $folderI18N = _("INBOX");
@@ -541,6 +639,9 @@ function proon_folder_blat_name($folder) {
   return $folderI18N;
 }
 
+/**
+ * Creates the email headers for the email post-pruning report.
+ */
 function proon_format_headers(&$p, &$outcome, $verbose) {
   $w = proon_calc_report_widths($p, $outcome, $verbose);
   include_once(SM_PATH . 'class/mime/Rfc822Header.class.php');
@@ -573,11 +674,28 @@ function proon_mailbox_list(&$p, &$imapStr) {
   return $p[PROON_P_MAILBOXES];
 }
 
+/**
+ * Utility for keeping track of subscribed folders.
+ */
 function proon_is_subscribed(&$p, &$target, $imapStr) {
   proon_mailbox_list($p, $imapStr);
   return (isset($p[PROON_P_BOXNAMES][$target]));
 }
 
+/**
+ * Utility for keeping track of folders which actually exist
+ */
+function proon_mailbox_exists(&$p, &$tgt, $imapStr) {
+  global $folder_prefix;
+  $target = $folder_prefix . $tgt;
+  if (proon_is_subscribed($p, $target, $imapStr)) return true;
+  $exists = sqimap_mailbox_exists($imapStr, $target);
+  return $exists;
+}
+
+/**
+ * Utility for logging time delta for some event.
+ */
 function proon_log_delta($tag, $t1, $t2 = null, $folder = null) {
   if ($t2 == null) $t2 = microtime();
   list($u1, $s1) = explode(' ', $t1);
@@ -587,13 +705,22 @@ function proon_log_delta($tag, $t1, $t2 = null, $folder = null) {
   proon_log($delta.' '.$tag, $folder);
 }
 
+/**
+ * Does arbitrary logging to a file for the squirrel_logger plugin.
+ */
 function proon_log($s, $folder = null) {
+  $f = ($folder ? "[$folder] " : '');
   global $proon_log_file;
-  if ($proon_log_file != null) {
-    $l = fopen($proon_log_file, 'a+');
-    $f = ($folder ? "[$folder] " : '');
-    fputs($l, date('Y-m-d H:i:s O').' ['.$_SESSION['username'].'] '.$f); fputs($l, $s); fputs($l, "\n");
-    fflush($l);
+  if ($proon_log_file == 'squirrel_logger') {
+    include_once(SM_PATH . 'plugins/squirrel_logger/functions.php');
+    sl_logit('PROON', $f.$s);
+  }
+  else {
+    if ($proon_log_file != null  &&  $proon_log_file !== false) {
+      $l = fopen($proon_log_file, 'a+');
+      fputs($l, date('Y-m-d H:i:s O').' ['.$_SESSION['username'].'] '.$f); fputs($l, $s); fputs($l, "\n");
+      fflush($l);
+    }
   }
 }
 
